import Agent from "../model/agentModel.js";
import multer from "multer";
import path from "path";
import crypto from "crypto";
import SibApiV3Sdk from "sib-api-v3-sdk";

// Helper to generate the Reference Number
const generateReferenceNumber = () => {
    return "AG-" + crypto.randomBytes(3).toString("hex").toUpperCase();
};

// ✅ Send Email Using Brevo API
const sendEmail = async (to, subject, params, templateId) => {
    try {
        if (!process.env.BREVO_API_KEY || !process.env.BREVO_EMAIL) {
            throw new Error("❌ [Brevo] Missing API Key or Sender Email in .env file!");
        }

        const client = SibApiV3Sdk.ApiClient.instance;
        const apiKey = client.authentications["api-key"];
        apiKey.apiKey = process.env.BREVO_API_KEY;

        const apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        const emailData = {
            sender: { email: process.env.BREVO_EMAIL },
            to: [{ email: to }],
            templateId: templateId,
            params: params,
        };

        await apiInstance.sendTransacEmail(emailData);
        console.log(`✅ [Brevo] Email sent to: ${to}`);
    } catch (error) {
        console.error("❌ [Brevo] Email Error:", error.response?.body || error);
    }
};

// Configure Multer Storage
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        console.log("Setting file destination...");
        cb(null, "uploads/");
    },
    filename: (req, file, cb) => {
        console.log("Generating filename for:", file.originalname);
        cb(null, Date.now() + path.extname(file.originalname));
    }
});

// Temporary: Allow any fields to debug issue
const upload = multer({ storage }).any();

// Register Agent Controller
export const registerAgent = async (req, res) => {
    console.log("Register agent called...");

    upload(req, res, async (err) => {
        if (err) {
            console.error("Multer Error:", err);
            return res.status(400).json({ error: err.message });
        }

        console.log("Received Body:", req.body);
        console.log("Received Files:", req.files);

        // Log received field names to detect mismatch
        const receivedFields = req.files.map(file => file.fieldname);
        console.log("Received Field Names:", receivedFields);
        console.log("Expected Field Names: ['image', 'profile_image', 'aadhar_image', 'vehicle_reg_image', 'vehicle_insurance_image']");

        try {
            // Normalize and fix incorrect key names
            let cleanedBody = {};
            Object.keys(req.body).forEach((key) => {
                cleanedBody[key.trim()] = req.body[key].trim();
            });

            // Correct key mapping
            cleanedBody.mobile = cleanedBody["mobile"] || cleanedBody["mobile "]; // Fix mobile field issue
            delete cleanedBody["mobile "]; // Remove incorrect key if exists

            console.log("Cleaned Body:", cleanedBody);

            const { city, name, email, mobile, vehicle_type, aadhar_no } = cleanedBody;

            // Validate Required Fields
            if (!city || !name || !email || !mobile || !vehicle_type || !aadhar_no) {
                console.error("Missing Required Fields");
                return res.status(400).json({ error: "All required fields must be filled" });
            }

            // Ensure at least one valid image is present
            if (!req.files.length) {
                console.error("Missing Required Images");
                return res.status(400).json({ error: "At least one image is required!" });
            }

            // Check for Duplicates
            console.log("Checking for duplicate agent...");
            const existingAgent = await Agent.findOne({ $or: [{ email }, { mobile }, { aadhar_no }] });

            if (existingAgent) {
                console.error("Duplicate Agent Found");
                return res.status(400).json({
                    error: "An agent with the same email, mobile, or Aadhar number already exists."
                });
            }

            // Extract File Paths
            const getFilePath = (fieldName) => {
                const file = req.files.find(file => file.fieldname === fieldName);
                return file ? file.path : null;
            };

            const profileImage = getFilePath("image") || getFilePath("profile_image");
            const aadharImage = getFilePath("aadhar_image");
            const vehicleRegImage = getFilePath("vehicle_reg_image");
            const vehicleInsuranceImage = getFilePath("vehicle_insurance_image");

            console.log("Profile Image Path:", profileImage);
            console.log("Aadhar Image Path:", aadharImage);
            console.log("Vehicle Reg Image Path:", vehicleRegImage);
            console.log("Vehicle Insurance Image Path:", vehicleInsuranceImage);

            // Save to MongoDB
            console.log("Saving agent to database...");
            const referenceNumber = generateReferenceNumber();

            const newAgent = new Agent({
                city,
                name,
                email,
                mobile,
                vehicle_type,
                aadhar_no,
                image: profileImage, // Profile image (Required)
                aadhar_image: aadharImage, // Optional
                vehicle_regd_image: vehicleRegImage, // Optional
                vehicle_insurance_image: vehicleInsuranceImage // Optional
            });

            await newAgent.save();
            console.log("Agent saved successfully!");

            // Send Email to Agent after successful registration
            const agentTemplateId = 47; // Agent Email Template ID
            await sendEmail(email, "Agent Registration Successful", { name, referenceNumber, currentYear: new Date().getFullYear() }, agentTemplateId);

            // Send Email to Admins after successful registration
            const adminTemplateId = 48; // Admin Email Template ID
            const adminEmails = [process.env.ADMIN_EMAIL_1, process.env.ADMIN_EMAIL_2]; // Use two admin emails
            for (let adminEmail of adminEmails) {
                await sendEmail(adminEmail, "New Agent Registration", { name, email, mobile, referenceNumber, currentYear: new Date().getFullYear() }, adminTemplateId);
            }

            // Return successful response
            res.status(201).json({
                message: "Agent registered successfully!",
                referenceNumber,
                agent: newAgent
            });

        } catch (error) {
            console.error("Error in registerAgent:", error);

            // Handle Duplicate Key Error
            if (error.code === 11000) {
                console.error("MongoDB Duplicate Key Error");
                return res.status(400).json({
                    error: "Duplicate entry detected. An agent with the same email, mobile, or Aadhar number already exists."
                });
            }

            res.status(500).json({ error: "Internal Server Error" });
        }
    });
};
