import DropLocation from "../model/dropLocation.js";
import SibApiV3Sdk from "sib-api-v3-sdk";

// Function to Send Email Using Brevo API
const sendEmail = async (to, subject, params, templateId) => {
    try {
        if (!process.env.BREVO_API_KEY || !process.env.BREVO_EMAIL) {
            throw new Error("❌ [Brevo] Missing API Key or Sender Email in .env file!");
        }

        const client = SibApiV3Sdk.ApiClient.instance;
        const apiKey = client.authentications["api-key"];
        apiKey.apiKey = process.env.BREVO_API_KEY;

        const apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        const emailData = {
            sender: { email: process.env.BREVO_EMAIL },
            to: [{ email: to }],
            templateId: templateId, // Dynamic Template ID passed to function
            params: params,
        };

        await apiInstance.sendTransacEmail(emailData);
        console.log(`✅ [Brevo] Email sent to: ${to}`);
    } catch (error) {
        console.error("❌ [Brevo] Email Error:", error.response?.body || error);
    }
};

// Function to generate a unique reference number
const generateReferenceNumber = () => {
    const timestamp = new Date().getTime();  // Get the current timestamp
    const randomNum = Math.floor(Math.random() * 1000);  // Generate a random number
    return `REF-${timestamp}-${randomNum}`;
};

export const registerDropLocation = async (req, res) => {
    try {
        // ✅ Match frontend keys to model fields
        const { shop_address, shop_name, shop_register_number, full_name, email, phone_number } = req.body;

        // Validation: Check Required Fields
        if (!shop_address || !shop_name || !shop_register_number || !full_name || !email || !phone_number) {
            return res.status(400).json({ error: "All fields are required!" });
        }

        // Check if email already exists
        const existingLocation = await DropLocation.findOne({ email });
        if (existingLocation) {
            return res.status(400).json({ error: "Email already exists!" });
        }

        // Generate a unique reference number for this drop location
        const reference_number = generateReferenceNumber();

        // Create and Save Drop Location with Reference Number
        const dropLocation = new DropLocation({
            shop_address,               // Matching to model field
            shop_name,                  // Matching to model field
            shop_register_number,       // Matching to model field
            full_name,                  // Matching to model field
            email,                      // Matching to model field
            phone_number,               // Matching to model field
            reference_number           // Add the generated reference number
        });

        await dropLocation.save();

        // ✅ Send Email to User after successful registration
        const userTemplateId = 45; // User Email Template ID
        await sendEmail(email, "Drop Location Registration Successful", 
            { shop_name, full_name, shop_address, shop_register_number, reference_number }, userTemplateId);

        // ✅ Send Email to Admins after successful registration
        const adminTemplateId = 46; // Admin Email Template ID
        const adminEmails = [process.env.ADMIN_EMAIL_1, process.env.ADMIN_EMAIL_2]; // Admin emails
        for (let adminEmail of adminEmails) {
            await sendEmail(adminEmail, "New Drop Location Registration", 
                { shop_name, full_name, email, phone_number, shop_address, shop_register_number, reference_number }, adminTemplateId);
        }

        return res.status(201).json({ message: "Drop location registered successfully!", dropLocation });

    } catch (error) {
        console.error("Error in registerDropLocation:", error);
        res.status(500).json({ error: "Internal Server Error" });
    }
};
