import Franchise from "../model/franchiseModel.js";
import SibApiV3Sdk from "sib-api-v3-sdk";
import crypto from "crypto";

// ✅ Function to Generate a Unique Reference Number
const generateReferenceNumber = () => {
    return "FR-" + crypto.randomBytes(3).toString("hex").toUpperCase();
};

// ✅ Send Email Using Brevo API
const sendEmail = async (to, subject, params, templateId) => {
    try {
        if (!process.env.BREVO_API_KEY || !process.env.BREVO_EMAIL) {
            throw new Error("❌ [Brevo] Missing API Key or Sender Email in .env file!");
        }

        const client = SibApiV3Sdk.ApiClient.instance;
        const apiKey = client.authentications["api-key"];
        apiKey.apiKey = process.env.BREVO_API_KEY;

        const apiInstance = new SibApiV3Sdk.TransactionalEmailsApi();
        const emailData = {
            sender: { email: process.env.BREVO_EMAIL },
            to: [{ email: to }],
            templateId: templateId, // Dynamic Template ID passed to function
            params: params,
        };

        await apiInstance.sendTransacEmail(emailData);
        console.log(`✅ [Brevo] Email sent to: ${to}`);
    } catch (error) {
        console.error("❌ [Brevo] Email Error:", error.response?.body || error);
    }
};

// ✅ Register a Franchise
export const registerFranchise = async (req, res) => {
    try {
        const { franchiseName, ownerName, email, phone, address, city } = req.body;

        // Restrict city from being 'Cuttack' or 'Bhubaneswar' (case insensitive)
        const restrictedCities = ["Cuttack", "Bhubaneswar"];
        if (restrictedCities.map(c => c.toLowerCase()).includes(city.trim().toLowerCase())) {
            return res.status(400).json({ message: `Franchises cannot be registered in ${city}.` });
        }

        // Check if email, phone already exists
        const existingFranchise = await Franchise.findOne({ 
            $or: [{ email }, { phone }] 
        });

        if (existingFranchise) {
            if (existingFranchise.email === email) {
                return res.status(400).json({ message: "Email already exists!" });
            }
            if (existingFranchise.phone === phone) {
                return res.status(400).json({ message: "Phone number already exists!" });
            }
        }

        // Generate Reference Number
        const referenceNumber = generateReferenceNumber();

        // Create a new Franchise entry
        const newFranchise = new Franchise({ franchiseName, ownerName, email, phone, address, city, referenceNumber });
        await newFranchise.save();

        // ✅ Send Email to User after successful registration
        const userTemplateId = 43; // User Email Template ID
        await sendEmail(email, "Franchise Registration Successful", { franchiseName, ownerName, referenceNumber, currentYear: new Date().getFullYear() }, userTemplateId);

        // ✅ Send Email to Admins after successful registration
        const adminTemplateId = 44; // Admin Email Template ID
        const adminEmails = [process.env.ADMIN_EMAIL_1, process.env.ADMIN_EMAIL_2]; // Use two admin emails
        for (let adminEmail of adminEmails) {
            await sendEmail(adminEmail, "New Franchise Registration", { franchiseName, ownerName, email, phone, referenceNumber, currentYear: new Date().getFullYear() }, adminTemplateId);
        }

        // Return successful response
        res.status(201).json({ message: "Franchise registered successfully!", referenceNumber, franchise: newFranchise });
    } catch (error) {
        res.status(500).json({ message: "Server Error", error: error.message });
    }
};
