import express from "express";
import dotenv from "dotenv";
import cors from "cors";
import multer from "multer";
import path from "path";
import fs from "fs";

import connectDb from "./db_conn.js";
import otpRoutes from "./routes/otpRoutes.js";
import agentRoutes from "./routes/agentRoutes.js";
import dropLocationRoutes from "./routes/dropLocationRoutes.js";
import franchiseRoutes from "./routes/franchiseRegisterRoutes.js";

dotenv.config();

// ✅ Attempt to connect to MongoDB
(async () => {
    try {
        await connectDb();
        console.log("✅ MongoDB Connected Successfully");
    } catch (error) {
        console.error("❌ MongoDB Connection Failed:", error.message);
        process.exit(1); // Stop server if DB connection fails
    }
})();

const app = express();
app.use(cors());
app.use(express.json());
app.use(express.urlencoded({ extended: true }));

// ✅ Serve uploaded images
app.use("/uploads", express.static("uploads"));

// ✅ Multer Storage Configuration
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        const uploadPath = "uploads/";
        if (!fs.existsSync(uploadPath)) {
            fs.mkdirSync(uploadPath, { recursive: true });
        }
        cb(null, uploadPath);
    },
    filename: (req, file, cb) => {
        cb(null, Date.now() + path.extname(file.originalname));
    }
});

// ✅ File filter to allow only images
const fileFilter = (req, file, cb) => {
    if (file.mimetype.startsWith("image/")) {
        cb(null, true);
    } else {
        cb(new Error("Only image files are allowed!"), false);
    }
};

// ✅ Configure Multer
const upload = multer({ storage, fileFilter });

// ✅ Route Definitions
app.use("/api/agent", agentRoutes);
app.use("/api/auth", otpRoutes);
app.use("/api/drop-location", dropLocationRoutes);
app.use("/api/franchise", franchiseRoutes);

// ✅ Global Error Handling Middleware
app.use((err, req, res, next) => {
    console.error("❌ Error:", err.message);
    res.status(err.status || 500).json({ error: err.message || "Internal Server Error" });
});

// ✅ Start Server
const PORT = process.env.PORT || 6999;
app.listen(PORT, () => console.log(`🚀 Server running on port ${PORT}`));
